/*
  Serial String Reader
  Language: Processing
  
  Reads in a string of characters until it gets a linefeed (ASCII 10). 
  Then converts the string into a number.  Then graphs it.
*/
import processing.serial.*;

int graphPosition = 0;  // the horizontal position of the latest
                        // line to be drawn on the graph
int linefeed = 10;    // linefeed in ASCII
Serial myPort;        // The serial port
int sensorValue = 0;  // the value from the sensor



void setup() {
  size(400,300);
  // List all the available serial ports
  println(Serial.list());

  // I know that the first port in the serial list on my Mac
  // is always my Arduino, so I open Serial.list()[0].
  // Open whatever port is the one you're using (the output
  // of Serial.list() can help; the are listed in order
  // starting with the one that corresponds to [0]).

  myPort = new Serial(this, Serial.list()[0], 9600);

  // read bytes into a buffer until you get a linefeed (ASCII 10):
  myPort.bufferUntil(linefeed);
}

void draw() {
  // twiddle your thumbs
}

// serialEvent  method is run automatically by the Processing applet
// whenever the buffer reaches the  byte value set in the bufferUntil() 
// method in the setup():

void serialEvent(Serial myPort) { 
  // read the serial buffer:
  String myString = myPort.readStringUntil(linefeed);

  // if you got any bytes other than the linefeed:
  if (myString != null) {
    //trim off the carriage return and convert the string to an integer:
    sensorValue = int(trim(myString));

    // print it:
    println(sensorValue);
    drawGraph();

  }
} 
void drawGraph() {
  // adjust this formula so that lineHeight is always less than
  // the height of the window:
  int lineHeight = sensorValue /2;

  // draw the line:
  stroke(0,255,0);
  line(graphPosition, height, graphPosition, height - lineHeight);

  // at the edge of the screen, go back to the beginning:
  if (graphPosition >= width) {
    graphPosition = 0;
    background(0); 
  } 
  else {
    graphPosition++;
  }
}
