/*
  RF Receive
  Language: Processing 
 
  This program listens for data coming in through a serial port. 
  It reads a string and throws out any strings that contain 
  values other than ASCII numerals, linefeed, or carriage return, 
  or that are longer than four digits. 
  This program is designed to work with a Laipac RF serial receiver 
  connected to the serial port, operating at 2400 bps.
*/
import processing.serial.*;

Serial myPort;            // the serial port
int incomingValue = 0;    // the value received in the serial port

void setup() {
  // list all the available serial ports:
  println(Serial.list());

  // open the appropriate serial port.  On my computer, the RF
  // receiver is connected to a USB-to-serial adaptor connected to 
  // the first port in the list. It may be on a different port on 
  // your machine:

  myPort = new Serial(this, Serial.list()[0], 2400); 
  // tell the serial port not to generate a serialEvent 
  //until a linefeed is received:
  myPort.bufferUntil('\n');
}

void draw() {
  // set the background color according to the incoming value:
  background(incomingValue/4);
}

// serialEvent  method is run automatically by the Processing applet
// whenever the buffer reaches the  byte value set in the bufferUntil() 
// method in the setup():

void serialEvent(Serial myPort) { 
  boolean validString = true;  // whether the string you got is valid
  String errorReason = "";     // a string that will tell what went wrong

  // read the serial buffer:
  String myString = myPort.readStringUntil('\n');

  // make sure you have a valid string:
  if (myString != null) {
    // trim off the whitespace (linefeed, carriage return) characters:
    myString = trim(myString);

    // check for garbage characters:
    for (int charNum = 0; charNum < myString.length(); charNum++) {
      if (myString.charAt(charNum) < '0' ||
         myString.charAt(charNum) > '9') {

        // you got a garbage byte; throw the whole string out
        validString = false; 
        errorReason = "Received a byte that's not a valid ASCII numeral.";
      } 
    }
    // check to see that the string length is appropriate:
    if (myString.length() > 4) {
      validString = false; 
      errorReason = "Received more than 4 bytes.";
    }

    // if all's good, convert the string to an int:
    if (validString == true) {
      incomingValue = int(trim(myString));
      println("Good value: " + incomingValue);
    } 
    else {
      // if the data is bad, say so:
      println("Error: Data is corrupted. " + errorReason);
    }
  }
} 
