/*
  Lantronix UDP Device Query
  Language: Processing

  Sends out a UDP broadcast packet to query a subnet for Lantronix 
  serial-to-ethernet devices.

  Lantronix devices are programmed to respond to UDP messages 
  received on port 30718.  If a Lantronix device receives the 
  string 0x00 0x00 0x00 0xF6, it responds with a UDP packet 
  containing the status message on port 30718.
  
  When the program starts, press any key on the keyboard and 
  watch the message pane for responses. 
  See the Lantronix integration guide from www.lantronix.com 
  for the details.

  This program uses the Hypermedia UDP library available at  
  hypermedia.loeil.org/processing/
*/
// import UDP library
import hypermedia.net.*;


UDP udp;                        // define the UDP object
int queryPort = 30718;          // the port number for the device query

void setup() {
  // create a new connection to listen for 
  // UDP datagrams on query port;
  udp = new UDP(this, queryPort);

  // listen for incoming packets:
  udp.listen( true );
}

//process events
void draw() {
  // twiddle your thumbs.  Everything is event-generated.
}

/*
 send the query message when any key is pressed:
 */
void keyPressed() {
  byte[] queryMsg = new byte[4];
  queryMsg[0] = 0x00;
  queryMsg[1] = 0x00;
  queryMsg[2] = 0x00;
  // because 0xF6 (decimal value 246) is greater than 128
  // you have to explicitly convert it to a byte:        
  queryMsg[3] = byte(0xF6);

  // send the message
  udp.send( queryMsg, "255.255.255.255", queryPort );
  println("UDP Query sent");
}

/*
  listen for responses via UDP
 */
void receive( byte[] data, String ip, int port ) {    
  String inString = new String(data);  // incoming data converted to string
  int[] intData = int(data);           // data converted to ints
  int i = 0;                           // counter
  
  // print the result:
  println( "response from "+ip+" on port "+port );

  // parse the response for the appropriate data.
  // if the fourth byte is <F7>, we got a status reply:
  print("Received response: ");
  println(hex(intData[3],2));
  if (intData[3] == 0xF7) {        
    // MAC address of the sender is bytes 24 to 30 (the end):
    print("MAC Addr: ");
    for (i=24; i < intData.length; i++) {
      print(" " + hex(intData[i], 2));
    }
  }
  // print a couple of blank lines to separate messages 
  // from multiple responders:
  print("\n\n");
}
