/*
Lantronix UDP Tester
Language: Processing
Sends and receives UDP messages from  Lantronix serial-to-ethernet devices.
Sends a serial message to a Lantronix device connected to the serial 
port when you type "s". Sends a UDP message to the Lantronix device 
when you type "u". Listens for both UDP and serial messages and 
prints them out.
*/
// import UDP library
import hypermedia.net.*;
// import serial library:
import processing.serial.*;

UDP udp;                          // define the UDP object
int queryPort = 10002;            // the port number for the device query
Serial myPort;
String xportIP = "192.168.1.20";  // fill in your Xport's IP here
int xportPort = 10001;            // the Xport's receive port
String inString = "";             // incoming serial string

void setup() {
  // create a new connection to listen for 
  // UDP datagrams on query port;
  udp = new UDP(this, queryPort);

  // listen for incoming packets:
  udp.listen( true );

  println(Serial.list());
  // make sure the serial port chosen here is the one attached 
  // to your Xport:
  myPort = new Serial(this, Serial.list()[0], 9600);
}

//process events
void draw() {
  // a nice blue background:
  background(0,0,255);
}

/*
 send messages when s or u key is pressed:
 */
void keyPressed() {
  switch (key) {
  case 'u':
    udp.send("Hello UDP!\r\n", xportIP, xportPort);
    break;
  case 's':
    String messageString = "Hello Serial!";

    for (int c = 0; c < messageString.length(); c++) {
      myPort.write(messageString.charAt(c)); 
    }
    break;
  }
}

/*
  listen for UDP responses
 */
void receive( byte[] data, String ip, int port ) {    
  String inString = new String(data); // incoming data converted to string
  println( "received "+inString +"  from "+ip+" on port "+port );

  // print a couple of blank lines to separate messages 
  // from multiple responders:
  print("\n\n");
}

/*
  listen for serial responses
 */
void serialEvent(Serial myPort) {
  // read any incoming bytes from the serial port and print them:
  char inChar = char(myPort.read());

  // if you get a linefeed, the string is ended; print it:
  if (inChar == '\n') {
    println("received " + inString + "  in the serial port\r\n"); 
    // empty the string for the next message:
    inString = "";
  } 
  else {
    // add the latest byte to inString:
    inString += inChar;
  }
}
