<?php

/*
toxic report
Language: PHP

This program opens a socket connection to an Xport and reads bytes from the socket. 
It then sorts the bytes into packets, interprets the packets, reports the results, 
and saves them  to a data log file.

*/

// Global variables.  These can be used by any of the script's functions:

global $ip, $port, $packetsToRead, $timeStamp, $messageString; 

$ip = "192.168.1.236";     // IP address to connect to
                          // Fill in the IP address of your Xport here

$port = 10001;            // port number of IP.
$packetsToRead = 10;      // total number of packets to read
$totalAverage = 0;        // the summary of all sensor readings
$packetCounter = 0;       // counter for packets when you're reading them
$bytes = array();         // array for bytes when you're reading them
$packets = array();       // array to hold the arrays of bytes

// $messageString is used to return messages for printing in the HTML:
$messageString = "No Sensor Reading Taken";

// Get the time and date:
$timeStamp =  $date = date("m-d-Y H:i:s");

/*--------------------------------------*/

//if a filled textbox was submitted, get the values:
if ((isset($_POST["ip"])) && 
    (isset($_POST["port"])) && 
    (isset($_POST["packetsToRead"]))) {
        $ip = $_POST["ip"];
        $port = $_POST["port"];
        $packetsToRead = $_POST["packetsToRead"];
}

/*--------------------------------------*/

//open a socket to the Xport:
$mySocket = fsockopen ($ip, $port, $errorno, $errorstr, 30); 
if (!$mySocket) { 
    //if the socket didn't open, return an error message
    return "Error $errorno: $errorstr<br>";
} else {    
    // if the socket exists, read packets until you've reached
    // $packetsToRead:
    while ($packetCounter < $packetsToRead) {
        // read a character from the socket connection,
        // and convert it to a numeric value using ord(),
        $char = ord(fgetc($mySocket));

        // if you got a header byte, deal with the last array
        // of bytes first:
        if ($char == 0x7E) {
            // push the last byte array onto the end of the packet array:
            array_push($packets, $bytes);    
            // clear the byte array:
            $bytes = array();
            // increment the packet counter:
            $packetCounter++;
        }               
        // push the current byte onto the end of the byte array:
        array_push($bytes, $char);
    }    
    // average the readings from all the packets to get a final
    // sensor reading:
    $totalAverage = averagePackets($packets);
    
    // update the message for the HTML:
    $messageString = 
      "Sensor Reading at:". $timeStamp . ": " . $totalAverage;

    // if you got a good reading, write it to the datalog file:
    if ($totalAverage > 0) {
        writeToFile($totalAverage);
    }
    //close the socket:
    fclose ($mySocket); 
}

/*--------------------------------------*/

function averagePackets($whichArray) {
    $packetAverage = 0;        // average of all the sensor readings
    $validReadings = 0;        // number of valid readings 
    $readingsTotal = 0;        // total of all readings, for averaging

  // iterate over the packet array:
    foreach ($whichArray as $thisPacket) {    
        // parse each packet to get the average sensor reading:
        $thisSensorReading = parsePacket($thisPacket);
        
        if ($thisSensorReading > 0 && $thisSensorReading < 1023) {
            // if the sensor reading is valid, add it to the total:
            $readingsTotal = $readingsTotal + $thisSensorReading;
            // increment the total number of valid readings:
            $validReadings++;
        }
    }
    if ($validReadings > 0) {    
        // round the packet average to 2 decimal points:
        $packetAverage = round($readingsTotal / $validReadings, 2);
        return $packetAverage;
    } else {
        return -1;
    }
}

/*--------------------------------------*/

function parsePacket($whichPacket) {    
    $adcStart = 11;                   // ADC reading starts at 12th byte
    $numSamples = $whichPacket[8];    // number of samples in the packet
    $total = 0;                       // sum of ADC readings for averaging

    // if you got all the bytes, find the average ADC reading:
    if( count($whichPacket) == 22) {
        // read the address. It's a two-byte value, so you
        // add the two bytes as follows:
        $address = $whichPacket[5] + $whichPacket[4] * 256;

        // read $numSamples 10-bit analog values, two at a time
        // because each reading is two bytes long:
        for ($i = 0; $i < $numSamples * 2;  $i=$i+2) {
            // 10-bit value = high byte * 256 + low byte:
            $thisSample = ($whichPacket[$i + $adcStart] * 256) + 
              $whichPacket[($i + 1) + $adcStart];
            // add the result to the total for averaging later:
            $total = $total + $thisSample;
        }
        // average the result:
        $average = $total / $numSamples;
        return $average;
    }  else {
        return -1;    
    }
}  

/*--------------------------------------*/

function writeToFile($whichReading) {
    global $timeStamp, $messageString;
    
    // combine the reading and the timestamp:
    $logData = "$timeStamp $whichReading\n";
    $myFile = "datalog.txt";    // name of the file to write to:
    
    // check to see that the file exists and is writable:
    if (is_writable($myFile)) {
        // try to write to the file:
       if (!($fh = fopen($myFile, "a"))) {
            $messageString = "Couldn't open file $myFile";
        } else {
            // if you could open the file but not write to it, say so:
            if (!fwrite($fh, $logData)) {
                $messageString = "Couldn't write to $myFile";
            }
        }
    } else {
        //if it's not writeable:
       $messageString = "The file $myFile is not writable";
    }
}
        
?>

    <html>
    <head>
   </head> 

    <body>
        <h2>
        <?=$messageString?>
        </h2>
        <hr>
        
        <form name="message" method="post" action="toxic_report.php">
            IP Address: <input type="text" name="ip" value="<?=$ip?>"
                         size="15" maxlength="15">
            Port: <input type="text" name="port" value="<?=$port?>" 
                   size="5" maxlength="5">  <br>
            Number of readings to take: <input type="text" 
              name="packetsToRead" value="<?=$packetsToRead?>" size="6">
            <input type="submit" value="Send It">
        </form>

    </body>
</html>