/*
  ID Innovations RFID Reader
  Language: Processing

  Reads data serially from an ID Innovations ID12 RFID reader.
*/

// import the serial library:
import processing.serial.*;

Serial myPort;      // the serial port you're using
String tagID = "";  // the string for the tag ID

void setup() {
  size(600,200);
  // list all the serial ports:
  println(Serial.list());

  // based on the list of serial ports printed from the 
  // previous command, change the 0 to your port's number:
  String portnum = Serial.list()[0];
  // initialize the serial port:
  myPort = new Serial(this, portnum, 9600);
  // incoming string from reader will have 16 bytes:
  myPort.buffer(16);

  // create a font with the second font available to the system:
  PFont myFont = createFont(PFont.list()[2], 24);
  textFont(myFont);
}

void draw() {
  // clear the screen:
  background(0); 
  // print the string to the screen:
  text(tagID, width/4, height/2 - 24);
}

/*
 this method reads bytes from the serial port
 and puts them into the tag string
 */
 
void serialEvent(Serial myPort) {
  // get the serial input buffer in a string:
  String inputString = myPort.readString();
  // filter out the tag ID from the string:
  tagID = parseString(inputString);
}

/*
  This method reads a string and looks for the 10-byte
 tag ID. It assumes it should get a STX byte (0x02) 
 at the beginning and an ETX byte (0x03) at the end
 */
String parseString(String thisString) {
  String tagString = "";    // string to put the tag ID into
  
  // first character of the input:
  char firstChar = thisString.charAt(0);  
  // last character of the input:
  char lastChar = thisString.charAt(thisString.length() -1);
  
  // if the first char is STX (0x02) and the last char
  // is ETX (0x03), then put the next ten bytes
  // into the tag string:
  
  if ((firstChar == 0x02) && (lastChar == 0x03)) {
    tagString = thisString.substring(1, 11);
  }
  return tagString;
}
