/*
  QRcode 2D Barcode Reader
  Language: Processing
  Uses the qrcode library from www.shiffman.net/p5/pqrcode based 
  on a Java library from  qrcode.sourceforge.jp. To use this, 
  generate images from a QRcode generator such as  qrcode.kaywa.com 
  and put them in this sketch's data folder.
  Press spacebar to read from the camera, generate an image, and 
  scan for barcodes.  Press f to read from a file and scan. 
  Press s for camera settings.
 
  by Tom Igoe / Daniel Shiffman
*/
 
import processing.video.*;
import pqrcode.*;

Capture video;                              // Video capture object
String statusMsg = "Waiting for an image";  // a string for messages:

// Decoder object from pqrdecoder library
Decoder decoder;

// make sure to generate your own image here:
String testImageName = "qrcode.png";

void setup() {
  size(400, 320);
  video = new Capture(this, width, height-20, 30);
  // Create a decoder object
  decoder = new Decoder(this);

  // Create a font with a font available to the system:
  PFont myFont = createFont(PFont.list()[2], 14);
  textFont(myFont);
}


void draw() {
  background(0);

  // Display video
  image(video, 0, 0);
  // Display status
  text(statusMsg, 10, height-4);

  // If we are currently decoding
  if (decoder.decoding()) {
    // Display the image being decoded
    PImage show = decoder.getImage();
    image(show,0,0,show.width/4,show.height/4); 
    statusMsg = "Decoding image";
    // fancy code for drawing dots as a progress bar:
    for (int i = 0; i < (frameCount/2) % 10; i++) 
    { 
      statusMsg += ".";
    }
  }
}

void captureEvent(Capture video) {
  video.read();
}

void keyReleased() {
  String code = "";
  // Depending on which key is hit, do different things:
  switch (key) {
  case ' ':        // Spacebar takes a picture and tests it:
    // copy it to the PImage savedFrame:
    PImage savedFrame = createImage(video.width,video.height,RGB);
    savedFrame.copy(video, 0,0,video.width,video.height,0,0,
      video.width,video.height);
    savedFrame.updatePixels();
    // Decode savedFrame
    decoder.decodeImage(savedFrame);
    break;
  case 'f':    // f runs a test on a file
    PImage preservedFrame = loadImage(testImageName);
    // Decode file
    decoder.decodeImage(preservedFrame);
    break;
  case 's':      // s opens the settings for this capture device:
    video.settings();
    break;
  }
}

// When the decoder object finishes
// this method will be invoked.
void decoderEvent(Decoder decoder) {
  statusMsg = decoder.getDecodedString(); 
}
