/*
RFIDto-X10 translator
 Language: Arduino
Reads RFID tags and sends X10 messages in response to the tags.
*/

// include the X10 library files:
#include <x10.h>
#include <x10constants.h>

#define zcPin 9         // the zero crossing detect pin
#define dataPin 8       // the X10 data out pin
#define repeatTimes 1   // how many times to repeat each X10 message
                        // in an electrically noisy environment, you  
                        // can set this higher.

#define tagLength 10    // each tag ID contains 10 bytes
#define startByte 0x0A  // for the ID Innovations reader, use 0x02
#define endByte 0x0D    // for the ID Innovations reader, use 0x03
#define dataRate 2400   // for the ID Innovations reader, use 9600

// set up a new x10  library instance:
x10 myHouse =  x10(zcPin, dataPin);

char tagID[tagLength];          // array to hold the tag you read
int tagIndex = 0;               // counter for number of bytes read
int tagComplete = false;        // whether the whole tag's been read
char tagOne[] = "0415AB6FB7";   // put the values for your tags here
char tagTwo[] = "0415AB5DAF";    
char lastTag = 0;               // value of the last tag read    
       
void setup() {
  // begin serial:
  Serial.begin(dataRate);
  // Turn off all lights:
  myHouse.write(A, ALL_LIGHTS_OFF,repeatTimes);
}
void loop() {
  // read in and parse serial data:
  if (Serial.available() > 0) {
    readByte();
  }

  // if you've got a complete tag, compare your tag
  // to the existing values:

  if (tagComplete == true) {
    if (compareTags(tagID, tagOne) == true) {

      if (lastTag != 1) {
        // if the last tag wasn't this one, 
        // send commands:
        myHouse.write(A, UNIT_1,repeatTimes);               
        myHouse.write(A, ON,repeatTimes);     
        myHouse.write(A, UNIT_2,repeatTimes);            
        myHouse.write(A, OFF,repeatTimes);
        // note that this was the last tag read:
        lastTag = 1;
      }
    }
    if (compareTags(tagID, tagTwo) == true) {
      if (lastTag != 2) {
        // if the last tag wasn't this one, 
        // send commands:
        myHouse.write(A, UNIT_1,repeatTimes);
        myHouse.write(A, OFF,repeatTimes);
        myHouse.write(A, UNIT_2,repeatTimes);              
        myHouse.write(A, ON,repeatTimes);
        // note that this was the last tag read:
        lastTag = 2;
      }
    }
  }
}

/*
  This method compares two char arrays byte by byte:
 */
char compareTags(char* thisTag, char* thatTag) {
  char match = true;  // whether they're the same

    for (int i = 0; i < tagLength; i++) {
    // if any two bytes don't match, the whole thing fails:
    if (thisTag[i] != thatTag[i]) {
      match = false; 
    }
  } 
  return match;
}

/*
  This method reads the bytes, and puts the 
 appropriate ones in the tagID
 
 */
void readByte() {
  char thisChar = Serial.read();

  switch (thisChar) {
  case startByte:     // start character
    // reset the tag index counter
    tagIndex = 0; 
    break;
  case endByte:    // end character 
    tagComplete = true;  // you have the whole tag
    break;
  default:    // any other character
    tagComplete = false;  // there are still more bytes to read
    // add the byte to the tagID
    if (tagIndex < tagLength) {
      tagID[tagIndex] = thisChar; 
      // increment the tag byte counter
      tagIndex++;
    }
    break;
  }
}
